<?php if (!defined('PmWiki')) exit();
/*
	The cmslike.php script makes PmWiki behave as a CMS.
	It complements UserAuth by presenting only the actions the current user is
	allowed to perform. It works with a single template, by inserting a dynamic
	actions menu as a variable, and disables the edit links in the text whenever
	editing is not allowed.

	Version 0.2 (development version; works with pmwiki-2.0.beta25 or above)

	Copyright 2005 Didier Lebrun (dl@vaour.net)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Requirements:
	* userauth.php MUST be loaded
	* userauth-0.6.php or above is needed

	Usage:
	* copy cmslike.php in the cookbook directory
	* add the following lines in your local/config.php file and adjust them to your taste:
		#$CmsLikeMenuItems = array(
		#	'print' => '<a href="$PageUrl?action=print" target="_blank" rel="nofollow">$[Printable View]</a>',
		#	'history' => '<a href="$PageUrl?action=diff" rel="nofollow">$[Page History]</a>',
		#	'edit' => '<a href="$PageUrl?action=edit" rel="nofollow">$[Edit Page]</a>',
		#	'upload' => '<a href="$PageUrl?action=upload" rel="nofollow">$[Upload]</a>',
		#	'attr' => '<a href="$PageUrl?action=attr" rel="nofollow">$[Attributes]</a>',
		#	'admin' => '<a href="$PageUrl?action=admin" rel="nofollow">$[UserAuth Admin]</a>',
		#	'pwchange' => '<a href="$PageUrl?action=pwchange" rel="nofollow">$[Change Password]</a>') );
		#$CmsLikeMenuSep = ' ';
		#$CmsLikeAltMenuItems = $CmsLikeMenuItems;
		#$CmsLikeAltMenuSep = ' - ';
		include_once('cookbook/cmslike.php'); # the module itself (you need this line at least !)
	* in your template:
		* replace your static actions menu with <!--function:CmsLikeMenuFmt--> for the standard menu setup
		  ... or with <!--function:CmsLikeMenuFmt alt--> for the alternate menu setup

	Notes:
	* UserAuth interception is used for calling CmsLikeMakeActionsList when the
	group is known, so as to establish the relevant rights. The conditions
	reproduce UserAuth ones.
	* <!--function:CmsLikeMenuFmt[ alt]--> in the template is used to evaluate
	the variables when they are fully defined.
	* You can use the "alt" optional arg to get the alternate menu setup instead
	of the standard (default) one, so you can have 2 different layouts in the
	same page (ex: a vertical one and an horizontal one like those in PmWiki
	default template)

	History:
	* 0.1	- Initial version

	* 0.2	- Change $_SESSION[...] calls to $UserInstanceVars->... (see userauth-0.6 changes)
			- Added page level auth conditions (see userauth-0.6 changes)
			- Rewrote auth processing to be cleaner and more adaptative to changes
			- Added Admin Tool and Change Password actions (see userauth-admintool and userauth-pwchange)
			- Change the template tags to <!--function:CmsLikeMenuFmt[ alt]--> (adjust your template !)
			- Fixed a potential bug when the configured $CmsLikeMenuItems lacks some actions
			- Change $AuthFunction interception so has to intercept only the first call

*/

if (!defined('USER_AUTH_VERSION')) exit();
define(CMSLIKE_VERSION, '0.2');

# You can override those in your local/config.php
SDV($CmsLikeMenuItems, array(
	'print' => '<a href="$PageUrl?action=print" target="_blank" rel="nofollow">$[Printable View]</a>',
	'history' => '<a href="$PageUrl?action=diff" rel="nofollow">$[Page History]</a>',
	'edit' => '<a href="$PageUrl?action=edit" rel="nofollow">$[Edit Page]</a>',
	'upload' => '<a href="$PageUrl?action=upload" rel="nofollow">$[Upload]</a>',
	'attr' => '<a href="$PageUrl?action=attr" rel="nofollow">$[Attributes]</a>',
	'admin' => '<a href="$PageUrl?action=admin" rel="nofollow">$[UserAuth Admin]</a>',
	'pwchange' => '<a href="$PageUrl?action=pwchange" rel="nofollow">$[Change Password]</a>') );
SDV($CmsLikeMenuSep, ' ');
# Alternate menu layout, so you can have 2 different menu layouts in your page
SDV($CmsLikeAltMenuItems, $CmsLikeMenuItems);
SDV($CmsLikeAltMenuSep, ' - ');

# Intercept AuthFunction calls
$AuthFunction = "CmsLikeAuth";

# Standard level => actions mapping
$CmsLikeActionsList = array('print');
$CmsLikeLevelToActions = array(
	'admin' => array('history', 'edit', 'upload', 'attr'),
	'edit' => array('history', 'edit'),
	'upload' => array('upload'),
	'attr' => array('attr') );

# AdminTool extension
if (defined('UA_ADMINTOOL'))
	array_push($CmsLikeLevelToActions['admin'], 'admin');

# PwChange extension
if (defined('UA_PWCHANGE')) {
	$CmsLikeLevelToActions['pwchange'] = array('pwchange');
	array_push($CmsLikeLevelToActions['admin'], 'pwchange');
}

function CmsLikeAuth($pagename, $level, $authprompt) {
	global $AuthFunction;

	# We build the list on the first AuthFunction call
	CmsLikeMakeActionsList($pagename);
	# ... then we let the calls go directly to UserAuth()
	$AuthFunction = "UserAuth";

	# Redirect to the real UserAuth function
	return UserAuth($pagename, $level, $authprompt);
}

function CmsLikeMakeActionsList($pagename) {
	global $GuestUsername, $LoggedInUsername, $UserInfoObj, $UserInstanceVars,
		$CmsLikeLevelToActions, $LinkPageCreateFmt, $CmsLikeActionsList;

	# Disables the create links in the text
	$no_create_link_fmt = "<a class='createlinktext'>\$LinkText</a><a class='createlink'>?</a>";

	if (isset($UserInfoObj)) {
		$username = $UserInstanceVars->GetInstanceUsername();
		if ($username=="") { $username = $GuestUsername; }
		$group = FmtPageName('$Group', $pagename);
		$name = FmtPageName('$Name', $pagename);

		# Auth processing is identical to UserAuth one
		foreach($CmsLikeLevelToActions as $level=>$actions) {
			$userauth_level = CmsLikeUserAuthLevel($level, $group, $name);
			if ( $UserInfoObj->UserHasAbility($username, $userauth_level)
					||  $UserInfoObj->UserHasAbility($GuestUsername, $userauth_level)
					||  $UserInfoObj->UserHasAbility($LoggedInUsername, $userauth_level)
					||  $UserInfoObj->UserHasAbility($username, $level.'_all') ) {
				foreach($actions as $action)
					if (!in_array($action, $CmsLikeActionsList))
						array_push($CmsLikeActionsList, $action);
			}
			if ( count($CmsLikeActionsList) == 1 + count($CmsLikeLevelToActions['admin']) ) break;
		}
	}
	else  $LinkPageCreateFmt = $no_create_link_fmt;
}

#
function CmsLikeUserAuthLevel($level, $group, $name) {
	global $UserInfoObj;

	$userauth_level = $level;

	$group_specific_level = $level."_group-".$group;
	if( $UserInfoObj->AnyUserHasAbility($group_specific_level) )
		$userauth_level = $group_specific_level;

	$page_specific_level = $level."_page-".$name;
	if( $UserInfoObj->AnyUserHasAbility($page_specific_level) )
		$userauth_level = $page_specific_level;

	return $userauth_level;
}

# Called by the template with <!--function:CmsLikeMenuFmt[ alt]-->
# "alt" is an optional parameter, for sending the alternate menu instead of the standard one
function CmsLikeMenuFmt($pagename, $alt="") {
	global $CmsLikeActionsList;
	$items_fmt = array();

	# Alternate menu
	if ($alt=='alt') {
		global $CmsLikeAltMenuItems, $CmsLikeAltMenuSep;

		foreach($CmsLikeActionsList as $action) {
			if (isset($CmsLikeAltMenuItems[$action]))
				array_push($items_fmt, $CmsLikeAltMenuItems[$action]);
		}
		PrintFmt($pagename, implode("\n".$CmsLikeAltMenuSep, $items_fmt));

	# Standard menu
	} else {
		global $CmsLikeMenuItems, $CmsLikeMenuSep;

		foreach($CmsLikeActionsList as $action) {
			if (isset($CmsLikeMenuItems[$action]))
				array_push($items_fmt, $CmsLikeMenuItems[$action]);
		}
		PrintFmt($pagename, implode("\n".$CmsLikeMenuSep, $items_fmt));
	}
}

?>
